<?php
namespace Misuzu;

use RuntimeException;

$authInfo = $msz->getAuthInfo();
if(!$authInfo->getPerms('user')->check(Perm::U_NOTES_MANAGE))
    Template::throwError(403);

$hasNoteId = filter_has_var(INPUT_GET, 'n');
$hasUserId = filter_has_var(INPUT_GET, 'u');

if((!$hasNoteId && !$hasUserId) || ($hasNoteId && $hasUserId))
    Template::throwError(400);

$urls = $msz->getURLs();
$usersCtx = $msz->getUsersContext();
$modNotes = $usersCtx->getModNotes();

if($hasUserId) {
    $isNew = true;

    try {
        $userInfo = $usersCtx->getUserInfo(filter_input(INPUT_GET, 'u', FILTER_SANITIZE_NUMBER_INT));
    } catch(RuntimeException $ex) {
        Template::throwError(404);
    }

    $authorInfo = $authInfo->getUserInfo();
} elseif($hasNoteId) {
    $isNew = false;

    try {
        $noteInfo = $modNotes->getNote((string)filter_input(INPUT_GET, 'n', FILTER_SANITIZE_NUMBER_INT));
    } catch(RuntimeException $ex) {
        Template::throwError(404);
    }

    if($_SERVER['REQUEST_METHOD'] === 'GET' && filter_has_var(INPUT_GET, 'delete')) {
        if(!CSRF::validateRequest())
            Template::throwError(403);

        $modNotes->deleteNotes($noteInfo);
        $msz->createAuditLog('MOD_NOTE_DELETE', [$noteInfo->getId(), $noteInfo->getUserId()]);
        Tools::redirect($urls->format('manage-users-notes', ['user' => $noteInfo->getUserId()]));
        return;
    }

    $userInfo = $usersCtx->getUserInfo($noteInfo->getUserId());
    $authorInfo = $noteInfo->hasAuthorId() ? $usersCtx->getUserInfo($noteInfo->getAuthorId()) : null;
}

while($_SERVER['REQUEST_METHOD'] === 'POST' && CSRF::validateRequest()) {
    $title = trim((string)filter_input(INPUT_POST, 'mn_title'));
    $body = trim((string)filter_input(INPUT_POST, 'mn_body'));

    if($isNew) {
        $noteInfo = $modNotes->createNote($userInfo, $title, $body, $authorInfo);
    } else {
        if($title === $noteInfo->getTitle())
            $title = null;
        if($body === $noteInfo->getBody())
            $body = null;

        if($title !== null || $body !== null)
            $modNotes->updateNote($noteInfo, $title, $body);
    }

    $msz->createAuditLog(
        $isNew ? 'MOD_NOTE_CREATE' : 'MOD_NOTE_UPDATE',
        [$noteInfo->getId(), $userInfo->getId()]
    );

    // this is easier
    Tools::redirect($urls->format('manage-users-note', ['note' => $noteInfo->getId()]));
    return;
}

Template::render('manage.users.note', [
    'note_new' => $isNew,
    'note_info' => $noteInfo ?? null,
    'note_user' => $userInfo,
    'note_user_colour' => $usersCtx->getUserColour($userInfo),
    'note_author' => $authorInfo,
    'note_author_colour' => $usersCtx->getUserColour($authorInfo),
]);
